namespace eval ton::lr {
    namespace export json2ton

    variable version 0.1
    
    proc json2ton json {lindex [jscan $json 0 [string length $json]] 1}
}
proc ton::lr::skipwhite {json i l} {
    while {[set c [string index $json $i]] eq "\n"
	   || [string is space $c]
	   &&  $i<$l
       } {incr i}
    return $i
}
proc ton::lr::jscan {json {i 0} {l -1} {d ",\}\]"}} {
    set i [skipwhite $json $i $l]
    switch -glob -- [string index $json $i] {
	[0-9-] {num $json $i $l $d}
	t {list [incr i 4] "l true"}
	f {list [incr i 5] "l false"}
	n {list [incr i 4] "l null"}
	\" {str $json [incr i] $l}
	[\[] {arr $json [incr i] $l}
	\{ {obj $json [incr i] $l}
	default {error "json string invalid: jscan [string range $json $i $i+10].."}
    }
}
proc ton::lr::num {json i l d} {
    set float false
    set c " "
    
    for {set j $i} {$i<$l} {incr i} {
	if {[set c [string index $json $i]] in $d} break
	set float [expr {$float || [string match "\[eE.]" $c]}]
    }
    set num [string range $json $j $i-1]
    if {$float} {    
	if {[string is double $num]} {
	    list $i "d $num"
	} else {
	    error "number: invalid double $num."
	}
    } elseif {[string is integer $num]} {
	list $i "i $num"
    } else {
	error "number: invalid integer $num."
    }
}
proc ton::lr::str {json i l} {
    for {set j $i} {$i<$l} {incr i} {
	set i [string first \" $json $i]
	if {[string index $json $i-1] ne "\\"} break
    }
    if {$i==$l} {
	error "json string invalid: exhausted while parsing string."
    }
    list [incr i] "s [list [string range $json $j $i-2]]"
}
proc ton::lr::arr {json i l} {
    set r a
    while {$i<$l} {
	lassign [jscan $json $i $l {, ]}] i v
	append r " " \[$v\]
	set i [skipwhite $json $i $l]
	switch -exact [string index $json $i] {
	    , {incr i}
	    \] {return [list [incr i] $r]}
	    default {
		error "json string invalid: parsing array."
	    }
	}
    }
}
proc ton::lr::obj {json i l} {
    set r o
    while {$i<$l} {
	lassign [jscan $json $i :] i k
	lassign $k type k
	if {$type ne "s"} {
	    error "json string invalid: key not a string."
	}
	set i [skipwhite $json $i $l]
	if {[string index $json $i] ne ":"} {
	    error "json string invalid: parsing key in object."
	}
	lassign [jscan $json [incr i] $l {, \}}] i v
	append r " " $k " " \[$v\]
	set i [skipwhite $json $i $l]
	switch -exact [string index $json $i] {
	    , {incr i}
	    \} {return [list [incr i] $r]}
	    default {
		error "json string invalid:$i: parsing object, [string index $json $i]."
	    }
	}
    }
}
